//s@pbP[W
#include "RunMethod/Posture.h"
#include "RunMethod/Vertical.h"
#include "RunMethod/Inclination.h"
#include "RunMethod/LineTraceMethod.h"
#include "RunMethod/RunMethod.h"
#include "RunMethod/PID.h"

//R[XpbP[W
#include "Course/RunCourse.h"
#include "Course/RunBlock.h"
#include "Course/Track.h"

//pbP[W
#include "Condition/AllDistanceCondition.h"
#include "Condition/Condition.h"

//⏕@\pbP[W
#include "SubFunction/Calibration.h"
#include "SubFunction/Stop.h"
#include "SubFunction/Sound.h"

//UIpbP[W
#include "UI/UI.h"

//ʐMpbP[W
#include "Communication/GetLogData.h"
#include "Communication/RemoteStart.h"

//API֘A
#include "Motor.h"
#include "TouchSensor.h"
#include "LightSensor.h"
#include "SonarSensor.h"
#include "GyroSensor.h"
#include "Clock.h"
#include "Bluetooth.h"
#include "Lcd.h"
#include "Daq.h"
#include "Speaker.h"
#include "Nxt.h"

using namespace ecrobot;


//------------------------------
//|[gݒ

const ePortS GYRO_SENSOR_PORT =			PORT_1;
const ePortS ULTRASONIC_SENSOR_PORT =	PORT_2;
const ePortS LIGHT_SENSOR_PORT =		PORT_3;
const ePortS TOUCH_SENSOR_PORT =		PORT_4;
const ePortM TAIL_MOTOR_PORT =			PORT_A;
const ePortM RIGHT_MOTOR_PORT =			PORT_B;
const ePortM LEFT_MOTOR_PORT =			PORT_C;



extern "C"
{

//JE^`
	DeclareCounter(SysTimerCnt);

//^XN`
	DeclareTask(RunTask);
	DeclareTask(UiTask);
	DeclareTask(EmergencyStopTask);

//A[`
	DeclareAlarm(RunAlarm);
	DeclareAlarm(UiAlarm);
	DeclareAlarm(EmergencyStopAlarm);

//Cxg`
	DeclareEvent(RunEvent);
	DeclareEvent(UiEvent);
	DeclareEvent(EmergencyStopEvent);

//*****************************************************************************
// ֐ 	: user_1ms_isr_type2
//  	: Ȃ
// ߂l	: Ȃ
// Tv 	: 1msec荞݃tbN֐(OSEK ISR type2JeS)
//*****************************************************************************

	void user_1ms_isr_type2(void)
	{
		SignalCounter(SysTimerCnt);		//JE^XV
		SleeperMonitor();				// needed for I2C devices and Clock class iClockNXŎgpj
	}


//API
	Motor tailMotor(TAIL_MOTOR_PORT);
	Motor leftMotor(LEFT_MOTOR_PORT);
	Motor rightMotor(RIGHT_MOTOR_PORT);
	TouchSensor touchSensor(TOUCH_SENSOR_PORT);
	LightSensor lightSensor(LIGHT_SENSOR_PORT);
	SonarSensor sonarSensor(ULTRASONIC_SENSOR_PORT);
	Clock clock;
	GyroSensor gyroSensor(GYRO_SENSOR_PORT);
	Bluetooth bluetooth;
	Lcd lcd;
	Daq daq(bluetooth);
	Speaker speaker;
	Nxt nxt;

//R[X
	RunCourse* in_basic;

//PID
	PID* pid_light;
	PID* pid_tail;

//K
	Posture* inc;

//|
	Posture* ver;

//TEh
	Sound* sound;

//m
	Condition* in01;
	Condition* in02;
	Condition* in03;

//s@
	RunMethod* run_straight;
	RunMethod* run_slope_up;
	RunMethod* run_slope_down;

//s
	RunBlock* start_straight;
	RunBlock* slope_up;
	RunBlock* slope_down;

//H
	Track* inbasic;

//̑
	RemoteStart* remotestart;
	UI* ui;
	Stop* st;
	Calibration* calibration;
	GetLogData* getlogdata;



//s^XNi4~bj
	TASK(RunTask)
	{

		//CX^X\z

		//PID
		pid_light = new PID(5.0,0.0,0.0);//P,I,D̏ԂōœKl͂ĂB
		pid_tail = new PID(4.5,0.0,0.0);

		//K
		inc = new Inclination(pid_tail,110,50,&tailMotor);

		getlogdata = new GetLogData(&daq);
		remotestart = new RemoteStart(&bluetooth);
		calibration = new Calibration(sound,&touchSensor,&lightSensor,&clock,&lcd,&gyroSensor);
		sound = new Sound(&speaker);
		ui = new UI(inc,remotestart,calibration,sound,&touchSensor,&clock,&tailMotor,&leftMotor,&rightMotor,&nxt,&bluetooth,&lcd);
		in_basic = new RunCourse(ui);
		inbasic = new Track();
		st = new Stop(&tailMotor,&leftMotor,&rightMotor,&gyroSensor,&speaker);
		ver = new Vertical(&tailMotor,&leftMotor,&rightMotor,&gyroSensor,&nxt);

		//INR[X̃x[VbŇmݒ
		in01 = new AllDistanceCondition(100,&leftMotor,&rightMotor);	//sF100cm
		in02 = new AllDistanceCondition(200,&leftMotor,&rightMotor);	//sF200cm
		in03 = new AllDistanceCondition(300,&leftMotor,&rightMotor);	//sF300cm

		//INR[X̃x[VbN̑s@ݒ
		run_straight	= new LineTraceMethod(calibration,ver,pid_light,20,50,&lightSensor,&leftMotor,&rightMotor);	//Cg[X̃Xs[hF20
		run_slope_up	= new LineTraceMethod(calibration,ver,pid_light,30,50,&lightSensor,&leftMotor,&rightMotor);	//Cg[X̃Xs[hF30
		run_slope_down	= new LineTraceMethod(calibration,ver,pid_light,40,50,&lightSensor,&leftMotor,&rightMotor);	//Cg[X̃Xs[hF40

		//INR[X̃x[VbN̑sݒ
		start_straight = new RunBlock(in01,run_straight,ui);
		slope_up  = new RunBlock(in02,run_slope_up,ui);
		slope_down  = new RunBlock(in03,run_slope_down,ui);

		//R[XɑHǉ
		in_basic->addTrack(inbasic);

		//inx[VbN̑s̒ǉ
		inbasic->addBlock(start_straight);
		inbasic->addBlock(slope_up);
		inbasic->addBlock(slope_down);

		//[^̃JE^NA
		leftMotor.setCount(0);
		rightMotor.setCount(0);
		tailMotor.setCount(0);

		//UI^XNN
		ActivateTask(UiTask);

		//ً}~^XNN
		ActivateTask(EmergencyStopTask);

		//s
		in_basic->prepareRun();

		while(1)
		{
			WaitEvent(RunEvent);		//sCxg҂
			ClearEvent(RunEvent);		//sCxgNA

			if(in_basic->startRun() == true)
			{
				tailMotor.setPWM(0);		// A[^ItbN֐FK
				leftMotor.setPWM(0);		// B[^ItbN֐FE[^
				rightMotor.setPWM(0);		// C[^ItbN֐F[^

				break;
			}
		}


		delete pid_light;
		delete pid_tail;
		delete ver;
		delete remotestart;
		delete ui;
		delete in_basic;
		delete inbasic;
		delete st;
		delete calibration;
		delete sound;
		delete inc;

		TerminateTask();				//^XNI
	}

	//UI^XNi1000~bj
	TASK(UiTask)
	{
		while(1)
		{
			WaitEvent(UiEvent);			//UICxg҂
			ClearEvent(UiEvent);		//UICxgNA
			ui->update();
			getlogdata->getLogData();

		}

		TerminateTask();

	}

	//ً}~^XNi1500~bj
	TASK(EmergencyStopTask)
	{

		while(1)
		{
			WaitEvent(EmergencyStopEvent);	//ً}~Cxg҂
			ClearEvent(EmergencyStopEvent);	//ً}~CxgNA

			st->stop();

		}

		TerminateTask();				//^XNI
	}

}


//Copyright (c) 2014 Kagaku No Yousei. All Rights Reserved.

